@@ -10,7 +10,18 @@ from pyqode.core.api import TextBlockHelper, folding, TextDecoration, \
 from pyqode.core.api.folding import FoldScope
 from pyqode.core.api.panel import Panel
 from pyqode.qt import QtCore, QtWidgets, QtGui, PYQT5_API
-from pyqode.core.api.utils import TextHelper, drift_color, keep_tc_pos
+from pyqode.core.api.utils import TextHelper, drift_color
+
+NAVIGATION_KEYS = (
+    QtCore.Qt.Key_Up,
+    QtCore.Qt.Key_Down,
+    QtCore.Qt.Key_Left,
+    QtCore.Qt.Key_Right,
+    QtCore.Qt.Key_Home,
+    QtCore.Qt.Key_End,
+    QtCore.Qt.Key_PageUp,
+    QtCore.Qt.Key_PageDown,
+)
 
 
 def _logger():
@@ -145,6 +156,10 @@ class FoldingPanel(Panel):
                         # this should never happen since we're working with
                         # clones
                         pass
+                    
+    @property
+    def fold_detector(self):
+        return self.editor.syntax_highlighter.fold_detector
 
     def __init__(self, highlight_caret_scope=False):
         Panel.__init__(self)
@@ -157,6 +172,7 @@ class FoldingPanel(Panel):
         )
         self._custom_color = QtGui.QColor('gray')
         self._block_nbr = -1
+        self._from_block = None
         self._highlight_caret = False
         self.highlight_caret_scope = highlight_caret_scope
         self._indic_size = 16
@@ -606,6 +622,7 @@ class FoldingPanel(Panel):
         """
         if state:
             self.editor.key_pressed.connect(self._on_key_pressed)
+            self.editor.key_released.connect(self._on_key_released)
             if self._highlight_caret:
                 self.editor.cursorPositionChanged.connect(
                     self._highlight_caret_scope)
@@ -613,6 +630,7 @@ class FoldingPanel(Panel):
             self.editor.new_text_set.connect(self._clear_block_deco)
         else:
             self.editor.key_pressed.disconnect(self._on_key_pressed)
+            self.editor.key_released.disconnect(self._on_key_released)
             if self._highlight_caret:
                 self.editor.cursorPositionChanged.disconnect(
                     self._highlight_caret_scope)
@@ -621,37 +639,29 @@ class FoldingPanel(Panel):
 
     def _on_key_pressed(self, event):
         """
-        Override key press to select the current scope if the user wants
-        to deleted a folded scope (without selecting it).
+        Override key press so that folded blocks are unfolded when they are
+        edited. This avoids inconsistencies when the edits actually change the
+        fold structure.
         """
-        delete_request = event.key() in [QtCore.Qt.Key_Backspace,
-                                         QtCore.Qt.Key_Delete]
-        if event.text() or delete_request:
-            cursor = self.editor.textCursor()
-            if cursor.hasSelection():
-                # change selection to encompass the whole scope.
-                positions_to_check = cursor.selectionStart(), cursor.selectionEnd()
-            else:
-                positions_to_check = (cursor.position(), )
-            for pos in positions_to_check:
-                block = self.editor.document().findBlock(pos)
-                th = TextBlockHelper()
-                if th.is_fold_trigger(block) and th.is_collapsed(block):
-                    self.toggle_fold_trigger(block)
-                    if delete_request and cursor.hasSelection():
-                        scope = FoldScope(self.find_parent_scope(block))
-                        tc = TextHelper(self.editor).select_lines(*scope.get_range())
-                        if tc.selectionStart() > cursor.selectionStart():
-                            start = cursor.selectionStart()
-                        else:
-                            start = tc.selectionStart()
-                        if tc.selectionEnd() < cursor.selectionEnd():
-                            end = cursor.selectionEnd()
-                        else:
-                            end = tc.selectionEnd()
-                        tc.setPosition(start)
-                        tc.setPosition(end, tc.KeepAnchor)
-                        self.editor.setTextCursor(tc)
+        
+        if event.key() in NAVIGATION_KEYS:
+            return
+        block = self.editor.textCursor().block()
+        if (
+            TextBlockHelper.is_collapsed(block) or
+            not block.isVisible()
+        ):
+            self._on_action_toggle()
+        self._from_block = block.blockNumber(), block.text()
+    
+    def _on_key_released(self, event):
+        
+        block = self.editor.textCursor().block()
+        if self.fold_detector and self.fold_detector.require_rehighlight(
+            self._from_block,
+            (block.blockNumber(), block.text())
+        ):
+            self.editor.syntax_highlighter.rehighlight()
 
     @staticmethod
     def _show_previous_blank_lines(block):
